/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : travel.cc                                                */
/*                                                                       */
/*       Date : November  6, 1997                                        */
/* Updated at : November 24, 1997                                        */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#include "p4.h"
#include "global.h"
#include "TSP_GA.h"

#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>

const int MINARGS = 10;

#define DOUBLE_MAX   1000000.0
#define INT_MAX       336666

void usage(char *progName)
{
  
  printf("Usage: %s \n",progName);
  printf("%20s %-20s %d )\n","popSize","(1 <= popSize <=",INT_MAX);
  printf("%20s %-20s %d )\n","generations","(1 <= generations <= %d",INT_MAX);
  printf("%20s %-20s\n","crossover","(0.0000 <= crossover <= 1.0000)");
  printf("%20s %-20s\n","crossoverType","(0 (= fixedPos),");
  printf("%20s %-20s\n\n","","1 (= fixedPosSeq))");
  printf("%20s %-20s\n\n","mutation","(0.0000 <= mutation  <= 1.0000)");
  printf("%20s %-20s\n","deletionType","(0 (= deleteAll),");
  printf("%20s %-20s\n\n","","1 (= steadyDelete))");
  printf("%20s %-20s\n","elitismValue","only used if deletionType = deleteAll");
  printf("%20s %-20s\n\n","","0 <= elitismValue <= popSize)");
  printf("%20s %-20s\n","fitnessType","(0 (= evaluation)");
  printf("%20s %-20s\n","","1 (= windowing)");
  printf("%20s %-20s\n\n","","2 (= linearNorm))");
  printf("%20s %-50s\n","fitnessParam","if fitnessType = evaluation");
  printf("%20s %-50s\n","","this is not used");
  printf("%20s %-50s\n","","if fitnessType = windowing");
  printf("%20s %-50s\n","","this is the guarding value");
  printf("%20s %-50s\n","","(1.0000 <= fitnessParam <= maxDist)");
  printf("%20s %-50s\n","","if fitnessType = linearNorm");
  printf("%20s %-50s\n","","this is the decrease value (%)");
  printf("%20s %-50s\n\n","","(0.0000 <= fitnessParam <= 100.0000)");
  printf("%20s %-20s %d )\n","seed","(0 (= random) <= seed <=", INT_MAX);
  //printf("[initfile]    (default value = genes.dat) \n");
  printf("%20s %s %f \n","Value limit","maxDist * genes * popSize <=",DOUBLE_MAX);   
  printf("%20s %-50s\n","","where  maxDist = the maximal Eucledian distance");
  printf("%20s %-50s\n","","between two genes in initfile");
  printf("%20s %-50s\n","","genes = number of genes in initfile + 1");

}

/*
//
*/

main(int argc, char *argv[])
{
  unsigned int chromoLength;

  char *progName;
  char *dataFile;
  char buf1[nameLength + 1];
  char buf2[nameLength + 1];
  char buf3[nameLength + 1];
  char buf[MAX_LINE_LENGTH];
  coord_t x,y;
  unsigned int i, j;
  type_Gene **genePool = NULL;
  type_TSP_GA *tspGA = NULL;

  int popSize;
  int generations;
  float crossover;
  enum cross_t crossoverType ;
  float mutation ;
  enum delete_t deletionType ;
  int elitismValue ;
  enum fitness_t fitnessType ;
  eval_t fitnessParam ;
  int seed ;

  if ((progName = strchr(argv[0], '/')) == (char *)0)
    progName = argv[0];
  else
    progName++;
  if (argc <= MINARGS)
    {
      printf("Error: Wrong number of arguments \n");
      usage(progName);
      return 1;
    }

  popSize = atoi(argv[1]);
  generations = atoi(argv[2]);
  crossover = atof(argv[3]);
  crossoverType = (enum cross_t)(atoi(argv[4]));
  mutation = atof(argv[5]);
  deletionType = (enum delete_t)(atoi(argv[6]));
  elitismValue = atoi(argv[7]);
  fitnessType = (enum fitness_t)(atoi(argv[8]));
  fitnessParam = atof(argv[9]);
  seed = atoi(argv[10]);
  
 

  printf(" argc = %d \n",argc);
  for(i=0;i<argc;i++)
      printf(" argv[%d] : %s \n",i, argv[i]);
  /***************************************************/
  /** If there is no data file, default is genes.dat */
  /***************************************************/
  //dataFile = (argc == (MINARGS + 1)?"genes.dat":argv[MINARGS + 1]);
  dataFile = (argc == (MINARGS + 1)?"genes.dat":argv[MINARGS + 1]);

  /*
	p4_initenv(&argc,argv);
	p4_create_procgroup();
 */

  tspGA = defineTSP_GA(dataFile,popSize,generations,
                                    crossover,crossoverType,mutation,
                                    deletionType, elitismValue, fitnessType, 
                                    fitnessParam,0);

  /**************/
  /** Read data */
  /**************/

  if(!readGenes(tspGA)) 
  {
      /*******************************/
      /** runs the srand             */
      /*******************************/
      /** correlation with ProcessID */
      /*******************************/
      setRandomSeed(tspGA,0); 
      printReport(tspGA);
      printGenePool(tspGA);

      for(i=0;i<3;i++)
      { 

         printf(" Run# %2d \n",i);
         runTimes(tspGA,generations);

         printf(" The Best Result is \n");
         printChromosome(getBestResult(tspGA));
         printf("\n"); 
         /*tspGA.printBestResult(cout);*/

	 /*tspGA.printPopulation(cout);
         //cout << "  After printPopulation " << endl;
         */
         /****************************************************
         /** Migration Time                                  *
         /**                                                 *
         /** Get the best from neighbor and replace with the *
         /** worst chromosome in the current population.     *
         /****************************************************/
         replaceWorstChromosomeinTSPWith(tspGA,getBestResult(tspGA));
         /*
         cout << "  After replaceWithWorst " << endl;
	 //tspGA.printPopulation(cout);
         //cout << "  After printPopulation " << endl;
         */

      }// end of if

  }// end of if

  /*p4_wait_for_end();*/
  return 0;

}// end of main
